# encoding: utf-8
"""
dataapi defines standard APIs for communicating with data service.

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from .data_api import DataApi

__all__ = ['DataApi']
