# encoding: utf-8
"""
tradeapi defines standard APIs for communicating with
algorithmic trading & executing system.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from .trade_api import TradeApi

__all__ = ['TradeApi']
